@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -42,8 +42,12 @@ from qutebrowser.misc import (miscwidgets, autoupdate, msgbox, httpclient,
 from qutebrowser.config import config, configfiles
 
 
-Result = enum.IntEnum('Result', ['restore', 'no_restore'],
-                      start=QDialog.Accepted + 1)
+class Result(enum.IntEnum):
+
+    """The result code returned by the crash dialog."""
+
+    restore = QDialog.Accepted + 1
+    no_restore = QDialog.Accepted + 2
 
 
 def parse_fatal_stacktrace(text):
@@ -158,6 +162,11 @@ class _CrashDialog(QDialog):
         self._init_info_text()
         self._init_buttons()
 
+    def keyPressEvent(self, e):
+        """Prevent closing :report dialogs when pressing <Escape>."""
+        if config.val.input.escape_quits_reporter or e.key() != Qt.Key_Escape:
+            super().keyPressEvent(e)
+
     def __repr__(self):
         return utils.get_repr(self)
 
@@ -192,7 +201,6 @@ class _CrashDialog(QDialog):
 
     def _init_checkboxes(self):
         """Initialize the checkboxes."""
-        pass
 
     def _init_buttons(self):
         """Initialize the buttons."""
@@ -293,7 +301,7 @@ class _CrashDialog(QDialog):
         self._paste_text = '\n\n'.join(lines)
         try:
             user = getpass.getuser()
-        except Exception as e:
+        except Exception:
             log.misc.exception("Error while getting user")
             user = 'unknown'
         try:
@@ -564,7 +572,6 @@ class ReportDialog(_CrashDialog):
 
     def _init_info_text(self):
         """We don't want an info text as the user wanted to report."""
-        pass
 
     def _get_error_type(self):
         return 'report'
